# -*- coding: utf-8 -*-
from datetime import datetime

from django import template
from django.utils.safestring import mark_safe

register = template.Library()


# Example: {{ '2014'|year_copyright }}
@register.filter
def year_copyright(start_year):
    start_year = int(start_year)
    curr_year = int(datetime.today().year)
    if start_year < curr_year:
        return '{0}-{1}'.format(start_year, curr_year)
    return start_year


@register.filter
def escape_double_brackets(value):
    return mark_safe(value.replace('"', '\\"'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
$$$$$___$$$$$$__$$$$$__$$__$$__$$___$$
$$__$$______$$__$$______$$$$___$$$_$$$
$$__$$______$$__$$$$_____$$____$$_$_$$
$$__$$__$$__$$__$$_______$$____$$___$$
$$$$$____$$$$___$$$$$____$$____$$___$$
"""
#
#  Convenient use of the Yandex map service for web development on 
#  the popular and free Django framework.
#
#  Copyright (c) 2014 genkosta
#
#  django-editor-ymaps is free software under terms of the MIT License.
#

from os import path
from setuptools import find_packages, setup


VERSION = (1, 2, 10)
__version__ = '.'.join(map(str, VERSION))


# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='django-editor-ymaps',
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    requires=['python (>= 3.5)', 'django (>= 2.0)'],
    description='Creating and editing Yandex maps.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='genkosta',
    author_email='genkosta43@gmail.com',
    url='https://github.com/genkosta/django-editor-ymaps',
    download_url='https://github.com/genkosta/django-editor-ymaps/tarball/master',
    license='MIT License',
    platforms=['any'],
    keywords=['django', 'yandex', 'maps', 'admin', 'editor'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=2.0',
        'Pillow',
        'django-imagekit',
        'python-slugify',
        'django-ckeditor',
        'lxml',
        'django-smart-selects==1.5.3',
        'django-ipware'
    ],
)



# -*- coding: utf-8 -*-
import random

from django.core.management import BaseCommand, CommandError
from django.utils.lorem_ipsum import paragraphs

from djeym.models import (CategoryPlacemark, CustomMarkerIcon, Map, Placemark,
                          SubCategoryPlacemark)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--count', type=int, default=100)

    def handle(self, *args, **options):
        count = options['count']

        if count == 0:
            msg = 'Error - The number of markers can not be zero. Parameter: --count'
            raise CommandError(msg)
        elif count < 0:
            msg = 'Error - The number of markers cannot be a negative number. Parameter: --count'
            raise CommandError(msg)

        slug = 'test'
        ymap = Map.objects.filter(slug=slug).first()

        if ymap is None:
            msg = 'Create a map with the name - Test'
            raise CommandError(msg)

        categories = CategoryPlacemark.objects.all()
        count_categories = categories.count()
        subcategories = SubCategoryPlacemark.objects.all()
        count_subcategories = subcategories.count()
        next_number_marker = Placemark.objects.count() + 1
        text = paragraphs(30, common=False)[0]
        icon_name_list = [icon.slug for icon in CustomMarkerIcon.objects.filter(icon_collection=ymap.icon_collection)]
        images = [
            '<p><img alt="" src="/media/uploads/2019/02/13/e51b6d5e-18df-4bb0-988a-b10b7a3bebb5.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/80c1d6e5-0a8e-48e5-9fda-980666ba2525.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/f05915cb-5683-4507-a186-c7f6466408ac.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/6be19160-164d-456d-aa3e-d407d40edae5.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/e988f41a-92cd-4f6c-b8db-8655188c1f2a.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/556e0572-e8f6-4b23-96ae-b53f73587ac9.jpg" style="height:214px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/da2fb316-9fd9-4d96-b4dd-7e44c0610eb2.jpg" style="height:181px; width:322px" /></p>',
            '<p><img alt="" src="/media/uploads/2019/02/13/36fa0a2f-6784-4314-a40b-903a1a734a35.png" style="height:120px; width:120px" /></p>'
        ]
        count_images = len(images)

        if count_categories == 0:
            msg = 'Create a category for markers!'
            raise CommandError(msg)

        for num in range(count):
            placemark = Placemark.objects.create(
                ymap=ymap,
                category=categories[random.randrange(count_categories)],
                header='<p>Marker heading - {}</p>'.format(next_number_marker),
                body='{0}<p>{1}</p>'.format(images[random.randrange(count_images)], text[:1000]),
                footer='<p>Footer text information - {}</p>'.format(next_number_marker),
                icon_name=random.choice(icon_name_list),
                coordinates='[{0},{1}]'.format(
                    random.randrange(-84, 76), random.randrange(-170, 170))
            )
            if count_subcategories > 0:
                placemark.subcategories.add(*[subcategories[idx] for idx in random.sample(
                    range(count_subcategories), random.randrange(count_subcategories) + 1)])
            next_number_marker += 1



# -*- coding: utf-8 -*-
import math
import os
import re
import shutil
import uuid
from decimal import Decimal
from io import BytesIO
from pathlib import Path

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db.models.fields.files import FileDescriptor, ImageFileDescriptor
from django.utils.translation import ugettext_lazy as _
from imagekit.models import ImageSpecField
from lxml import etree

# Maximum size of icons for markers and clusters of Yandex maps.
# (Максимальный размер иконок для маркеров и кластеров Яндекс карт.)
DJEYM_YMAPS_ICONS_MAX_SIZE = 60


def validate_image(image):
    extension_list = ['.jpg', '.jpeg', '.png']
    size = image.size
    extension = Path(image.name).suffix.lower()

    if extension not in extension_list:
        raise ValidationError(_('Only JPG or PNG format files.'))
    elif not size:
        raise ValidationError(_('Image cannot be 0.0 mb.'))
    elif not size or size > 524288:
        raise ValidationError(_('Maximum image size 0.5 mb.'))


def validate_hex_color(color):
    if re.match(r'^#[0-9a-f]{3,8}$', color, re.I) is None:
        raise ValidationError(_('Color does not match the hex standard.'))


def get_icon_font_plugin():
    """Get plug-in files for icon font."""
    icons_default_bool = False
    ymaps_icons_for_categories_css = getattr(
        settings, 'DJEYM_YMAPS_ICONS_FOR_CATEGORIES_CSS', None)
    ymaps_icons_for_categories_js = getattr(
        settings, 'DJEYM_YMAPS_ICONS_FOR_CATEGORIES_JS', None)
    if not bool(ymaps_icons_for_categories_css) or not bool(ymaps_icons_for_categories_js):
        icons_default_bool = True
        ymaps_icons_for_categories_css = [
            '/static/djeym/plugins/fontawesome/css/all.min.css']
        ymaps_icons_for_categories_js = [
            '/static/djeym/plugins/fontawesome/js/all.min.js']
    return ymaps_icons_for_categories_css, ymaps_icons_for_categories_js, icons_default_bool


def get_size_correction(width, height):
    """Get size correction of image."""
    size_width, size_height = Decimal(width), Decimal(height)

    if size_width > DJEYM_YMAPS_ICONS_MAX_SIZE or size_height > DJEYM_YMAPS_ICONS_MAX_SIZE:
        size_width = math.ceil(
            (size_width * (DJEYM_YMAPS_ICONS_MAX_SIZE / size_height)))
        size_height = DJEYM_YMAPS_ICONS_MAX_SIZE
    return [size_width, size_height]


def get_size_from_svg(svg):
    """Get width and height from SVG file."""
    with open(svg.path, mode="rb") as svg_file:
        xml_content = svg_file.read()
        root = etree.XML(xml_content)
        result = {
            "width": int(Decimal(root.get('width'))),
            "height": int(Decimal(root.get('height')))
        }
    return result


def validate_svg(svg):
    extension_list = ('.svg',)
    extension = Path(svg.name).suffix.lower()

    if extension not in extension_list:
        raise ValidationError(_('Only SVG (file_name.svg) files.'))

    if hasattr(svg, 'temporary_file_path'):
        svg_path = svg.temporary_file_path()
    else:
        if hasattr(svg, 'read'):
            svg_path = BytesIO(svg.read())
        else:
            svg_path = BytesIO(svg['content'])

    if bool(svg_path):
        xml_content = svg_path.read()
        try:
            root = etree.XML(xml_content)
        except etree.XMLSyntaxError:
            svg_path.close()
            raise ValidationError(_('This is an invalid SVG file. Open this file in '
                                    'the vector editor and try to fix it.'))
        tag = root.tag
        if bool(tag) and tag == "{http://www.w3.org/2000/svg}svg":
            if root.get('width') is not None and root.get('height') is not None:
                if root.get('viewBox') is None:
                    svg_path.close()
                    raise ValidationError(_('Missing viewBox attribute'))
            else:
                svg_path.close()
                raise ValidationError(
                    _('There are no width and height attributes in the SVG tag.'))
        else:
            svg_path.close()
            raise ValidationError(_('This is an invalid SVG file. Open this file in '
                                    'the vector editor and try to fix it.'))
    else:
        svg_path.close()
        raise ValidationError(
            _('Unable to read file attributes. The file may be damaged.'))


def get_errors_form(*args):
    err_dict = {}
    detail = ''

    for form in args:
        for item in list(form):
            if item.errors:
                err_dict[item.name] = True
                detail += u'{0}: {1}<br>'.format(item.label, item.errors)
            else:
                err_dict[item.name] = False

    return {'err_dict': err_dict, 'detail': detail}


def validate_coordinates(coordinate):
    try:
        float(coordinate)
    except ValueError:
        raise ValidationError(
            _('To determine the coordinates a numeric value is required.'))


def validate_transparency(coordinate):
    try:
        float(coordinate)
    except ValueError:
        raise ValidationError(
            _('To determine the transparency a numeric value is required.'))


def make_upload_path(instance, filename):
    extension = Path(filename).suffix
    return os.path.join(instance.upload_dir, '{0}{1}'.format(uuid.uuid4(), extension))


def get_filename(filename):
    """CKEditor - Filename Generator"""
    extension = Path(filename).suffix
    return '{0}{1}'.format(uuid.uuid4(), extension)


def cleaning_files_pre_save(sender, instance, **kwargs):
    """Remove old files"""
    old_status = sender.objects.filter(pk=instance.pk)
    dir_name_list = dir(sender)

    if old_status:
        for dir_name in dir_name_list:
            field = getattr(sender, dir_name, None)

            if field and (isinstance(field, FileDescriptor) or
                          isinstance(field, ImageFileDescriptor) or
                          isinstance(field, ImageSpecField)):

                old_field = getattr(old_status[0], dir_name)
                updated_field = getattr(instance, dir_name)

                if getattr(old_field, 'name', None) != getattr(updated_field, 'name', None):

                    try:
                        path = getattr(old_field, 'path')

                        if path and os.path.exists(path):
                            if isinstance(field, ImageSpecField):
                                pattern = re.compile(
                                    r'(/[-_\w()]+\.[a-zA-Z]{3,4}$)')
                                shutil.rmtree(pattern.sub('', path))
                            else:
                                os.remove(path)
                    except (FileNotFoundError, ValueError):
                        pass


def cleaning_files_pre_delete(sender, instance, **kwargs):
    """Delete orphan files"""
    dir_name_list = dir(sender)

    for dir_name in dir_name_list:
        field = getattr(sender, dir_name, None)

        if field and (isinstance(field, FileDescriptor) or
                      isinstance(field, ImageFileDescriptor) or
                      isinstance(field, ImageSpecField)):

            target_field = getattr(instance, dir_name)

            try:
                path = getattr(target_field, 'path', None)

                if path and os.path.exists(path):
                    if isinstance(field, ImageSpecField):
                        pattern = re.compile(r'(/[-_\w()]+\.[a-zA-Z]{3,4}$)')
                        shutil.rmtree(pattern.sub('', path))
                    else:
                        os.remove(path)
            except (FileNotFoundError, ValueError):
                pass
